﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Globalization;
using System.Collections;
using System.Collections.ObjectModel;
using System.Windows.Automation;

namespace TreeViewTriStateBinding
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window
	{
		public Window1()
		{
			InitializeComponent();

			treeView.ItemsSource = new ObservableCollection<MyViewModel>()
			{
				new MyViewModel
				{ 
					Title = "Item 1",
					Children = 
					{
						new MyViewModel
						{
							Title = "Child 1",
							IsChecked = true,
						},
						new MyViewModel
						{
							Title = "Child 2",
							IsChecked = false,
						}
					}
				},

				new MyViewModel
				{ 
					Title = "Item 2",
					Children = 
					{
						new MyViewModel
						{
							Title = "Child 3",
							IsChecked = true,
						},
						new MyViewModel
						{
							Title = "Child 4",
							IsChecked = null,
							Children = 
							{
								new MyViewModel
								{
									Title = "GrandChild 1",
									IsChecked = false,
								},
								new MyViewModel
								{
									Title = "GrandChild 2",
									IsChecked = true,
								}
							}
						}
					}
				},

				new MyViewModel
				{ 
					Title = "Item 3",
					IsChecked = true,
					Children = 
					{
						new MyViewModel
						{
							Title = "Child 5",
							IsChecked = true,
						},
						new MyViewModel
						{
							Title = "Child 6",
							IsChecked = true,
						}
					}
				}
			};
		}

	}

	public class CheckStateConverter : IValueConverter
	{
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			bool? isChecked = (bool?)value;

			if (isChecked == null)
			{
				return ToggleState.Indeterminate;
			}
			if (isChecked.Value)
			{
				return ToggleState.On;
			}
			return ToggleState.Off;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			switch ((ToggleState)value)
			{
				case ToggleState.Off:
					return false;
				case ToggleState.On:
					return true;
				default:
					return null;
			}
		}
	}

	public class MyViewModel : INotifyPropertyChanged
	{
		public MyViewModel()
		{
			this.Children = new ObservableCollection<object>();
		}

		public IList Children { get; set; }

		private bool? isChecked;
		/// <summary>
		///     Gets or sets whether the item is checked.
		/// </summary>
		public bool? IsChecked
		{
			get
			{
				return this.isChecked;
			}
			set
			{
				if (this.isChecked != value)
				{
					this.isChecked = value;
					OnPropertyChanged("IsChecked");
				}
			}
		}


		private String title;
		/// <summary>
		///     Gets or sets the name.
		/// </summary>
		public String Title
		{
			get
			{
				return this.title;
			}
			set
			{
				if (this.title != value)
				{
					this.title = value;
					OnPropertyChanged("Title");
				}
			}
		}

		/// <summary>
		///     Event for INotifyPropertyChanged.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		///     Raises the INotifyPropertyChanged event.
		/// </summary>
		/// <param name="propertyName">The property to be included in the INotifyPropertyChanged EventArgs.</param>
		protected virtual void OnPropertyChanged(String propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		public override string ToString()
		{
			return "VM " + title;
		}
	}
}
